/* global describe, it, beforeEach */
/* jslint node: true, esnext: true */

"use strict";

const chai = require('chai');
const assert = chai.assert;
const expect = chai.expect;
const should = chai.should();

const fs = require('fs');
const read = fs.createReadStream;
const path = require('path');
const eol = require('os').EOL;

const fixturesDir = path.join(__dirname, 'fixtures');

const dataProcessor = require('../index');
const testStep = require('kronos-test-step');
const step = require('kronos-step');
const messageFactory = require('kronos-message');

// ---------------------------
// Create a mock manager
// ---------------------------
const manager = testStep.managerMock;

dataProcessor.registerWithManager(manager);


describe('data-processor: simple import', function () {

	it('big file', function (done) {
		const config = require('./fixtures/demo_config.json');
		let inFile = path.join(fixturesDir, 'demo_import.csv');

		/**
		 * @param err All the errors if there where any
		 * @param rows The row objects
		 */
		function verify(err, rows) {
			assert.notOk(err);

			// the count of rows
			assert.equal(rows.length, 20);

			// just check one of the data packets
			assert.deepEqual(rows[0], {
				"lineNumber": 1,
				"data": {
					"app_id": "WURP",
					"account": "SAEDsa34",
					"server": "gum@org.de",
					"start-date": "27.02.1984 11:04:38",
					"technical": "y"
				}
			});
			done();
		}

		checkStep(inFile, config, verify);
	});

});

/**
 * This function creates a new step and passes a file reader
 * stream in a message to it
 * @param inputFile The demo input file
 * @param options The options to create the data processor
 * @param verifyFunction The function to verify the result
 */
function checkStep(inputFile, options, verifyFunction) {

	// Set the step name
	options.name = "myDataProcessor";
	options.type = "kronos-step-passThrough";

	// get the endpoints of the step
	let dataProcessor = manager.getStepInstance(options);
	let inEndPoint = dataProcessor.endpoints.inFileStream;
	let outEndPoint = dataProcessor.endpoints.outRowStream;

	// Stores the result rows
	var lines = [];

	// This endpoint is the OUT endpoint of the previous step.
	// It will be connected with the OUT endpoint of the Adpater
	let sendEndpoint = step.createEndpoint("testEndpointOut", {
		"out": true
	});

	// This endpoint is the IN endpoint of the next step.
	// It will be connected with the OUT endpoint of the Adpater
	let receiveEndpoint = step.createEndpoint("testEndpointIn", {
		"in": true
	});


	// This generator emulates the IN endpoint of the next step.
	// It will be connected with the OUT endpoint of the Adpater
	let receiveFunction = function (message) {

		// after the file was written we need to compare it
		message.payload.on('data', function (line) {
				lines.push(line);
			})
			.on('error', function (err) {
				verifyFunction(err, lines);
			})
			.on('end', function () {
				verifyFunction(false, lines);
			});
	};

	receiveEndpoint.receive = receiveFunction;
	outEndPoint.connect(receiveEndpoint);
	inEndPoint.connect(sendEndpoint);

	let msg = messageFactory({
		"file_name": inputFile
	});
	msg.payload = fs.createReadStream(inputFile);

	dataProcessor.start().then(function (step) {
		sendEndpoint.send(msg);
	}, function (error) {
		throw error; // 'uh oh: something bad happened’
	});

}
